
use <../Util.scad>


// TODO: documentation!!!!!!
module straightPipe(width, height, length, radius, strength, center=false, inner=true, outer=true) {
  w = width;
  h = height;
  l = length;
  r = radius;
  _str = strength;
  
  tz = center ? -l/2 : 0;
  
  translate([0, 0, tz]) linear_extrude(l) difference() {
    if (outer) roundRect(w, h, r, center=true);
    if (inner) roundRect(w - _str*2, h - _str*2, r - _str, center=true);
  }
}

// straightPipe(20, 10, 30, 1, 1);


// center only applies to Z-axis
// TODO: write documentation!
module cornerPipe(width, height, diameter, angle, radius, strength, center=false, inner=true, outer=true) {
  w = width;
  h = height;
  d = diameter;  // "size" of the corner. inner diameter of where it is virtually wound onto
  a = angle;
  r = radius;
  _str = strength;
  
  tz = center ? 0 : h/2;
  
  translate([0, 0, tz]) rotate_extrude(angle=a, convexity=8) translate([w/2 + d/2, 0, 0]) difference() {
    if (outer) roundRect(w, h, r, center=true);
    if (inner) roundRect(w - _str*2, h - _str*2, r - _str, center=true);
  }
}


// TODO: LayOnFace? Use a better name maybe…?
module cornerPipeLayOnFace(width, height, diameter, angle, radius, strength, center=false, centerX=false, centerY=false, inner=true, outer=true) {
  
  tx = center || centerX ? -height/2 : 0;
  ty = center || centerY ? -width/2 : 0;
  
  translate([tx + height, ty + width + diameter/2, 0]) rotate([90, 0, -90]) cornerPipe(width=width, height=height, diameter=diameter, angle=angle, radius=radius, strength=strength, center=center, inner=inner, outer=outer);
}

// cornerPipe(20, 10, 5, 90, 1, 1, true);


// Not what I wanted to code - but may come in handy anywhere in the future…
module helixPipe(width, height, length, diameter, angle, radius, strength, center=false, inner=true, outer=true) {
  w = width;
  h = height;
  l = length;
  a = angle;
  r = radius;
  _str = strength;
  
  tz = center ? 0 : h/2;
  
  translate([0, 0, tz]) linear_extrude(l, twist=a, convexity=8) translate([0, 0, 0]) difference() {
    if (outer) roundRect(w, h, r, center=true);
    if (inner) roundRect(w - _str*2, h - _str*2, r - _str, center=true);
  }
}

// helixPipe(20, 10, 20, 5, 90, 1, 1, true);


// TODO: documentation!
module adapterPipe(
  w0, h0, r0, str0, w1, h1, r1, str1,
  length, x_off=0, y_off=0,
  center=false, inner=true, outer=true
) {

  l = length;
  epsilon = 0.001;
  tz = center ? -l/2 : 0;
  
  difference() {
    // outer shell
    if (outer) hull() {
      linear_extrude(epsilon) roundRect(w0, h0, r0, center=true);
      translate([x_off, y_off, l - epsilon]) linear_extrude(epsilon) roundRect(w1, h1, r1, center=true);
    }

    // inner hole
    if (inner) hull() {
      linear_extrude(epsilon) roundRect(w0 - str0*2, h0 - str0*2, r0 - str0, center=true);
      translate([x_off, y_off, l - epsilon]) linear_extrude(epsilon) roundRect(w1 - str1*2, h1 - str1*2, r1 - str1, center=true);
    }
  }
}

// adapterPipe(20, 30, 6, 1,   20, 10, 1, 2,  30,  x_off = 23, center=true);


// N is the number of pipe endings
// length means length of each pipe measured from the centre
module crossNPipe(width, height, length, radius, strength, N, center=false, inner=true, outer=true) {
  w = width;
  h = height;
  l = length;
  r = radius;
  _str = strength;
  
  epsilon = 0.001;  // TODO: may need adjustment
  
  tz = center ? h/2 : 0;
  
  translate([0, 0, tz]) difference() {
    if (outer) for (rz=[0:360/N:360]) rotate([-90, 0, rz]) linear_extrude(l) {
      roundRect(w, h, r, center=true);
    }

    if (inner) for (rz=[0:360/N:360]) rotate([-90, 0, rz]) linear_extrude(l + epsilon) {
      roundRect(w - _str*2, h - _str*2, r - _str, center=true);
    }
  }
}

// crossNPipe(20, 10, 40, 1, 1, 6);

