
$E = 0.01;  // ε

assertDiameterMsg = "diameter out of range[ >= 0 ]";


/**
  * A round triangle pointing along the X-Axis
  *
  * `w` width; length along X
  * `h` height; Y size
  * `r` corner radius
  * `style` use "inner" for the included area or anything else for the `outer` area, this is the included area plus corner size
  *
  */
module roundTri(w, h, r, style="inner") {
  
  _r = style == "inner" ? r : 0;
  
  hull () {
    translate([_r, -h/2 + _r, 0]) circle(r=r);
    translate([_r, h/2 - _r, 0]) circle(r=r);
    translate([w - _r, 0, 0]) circle(r=r);
  }
}



/**
  *
  * TODO: Moar documentation!
  *
  */
module roundRect(w, h, r, center=false, centerX=false, centerY=false) {

  tx = center || centerX ? -w/2 : 0;
  ty = center || centerY ? -h/2 : 0;

  translate([tx, ty, 0])
    if (r > 0)
      hull() for (_tx=[r, w-r], _ty=[r, h-r]) translate([_tx, _ty, 0]) circle(r=r);
    else
      square([w, h]);
}


/**
  *
  * TODO: Moar documentation!
  *
  * verts = array[4] describing round edges
  *         every   0 will become right angled
  *         every > 0 will become rounded, number = radius
  *
  */
module mixedRoundRect(w, h, verts, center=false, centerX=false, centerY=false) {
  
  assert(len(verts) >= 4, "verts must have at least 4 elements");  // TODO: maybe fill with zero instead?

  w2 = w / 2;
  h2 = h / 2;

  tx = center || centerX ? 0 : w2;
  ty = center || centerY ? 0 : h2;


  v0  = verts[0];
  v1  = verts[1];
  v2  = verts[2];
  v3  = verts[3];

  // read: vert N is rounded
  v0r = v0 >= $E;
  v1r = v1 >= $E;
  v2r = v2 >= $E;
  v3r = v3 >= $E;


  tx0 = v0r ? -w2 + v0 : -w2;
  ty0 = v0r ? +h2 - v0 : +h2;

  tx1 = v1r ? -w2 + v1 : -w2;
  ty1 = v1r ? -h2 + v1 : -h2;

  tx2 = v2r ? +w2 - v2 : +w2;
  ty2 = v2r ? -h2 + v2 : -h2;

  tx3 = v3r ? +w2 - v3 : +w2;
  ty3 = v3r ? +h2 - v3 : +h2;


  translate([tx, ty, 0]) hull() {

    translate([tx0, ty0, 0])
      if (v0r) circle(r=v0);
      else square($E, center=true);
    
    translate([tx1, ty1, 0])
      if (v1r) circle(r=v1);
      else square($E, center=true);
    
    translate([tx2, ty2, 0])
      if (v2r) circle(r=v2);
      else square($E, center=true);
    
    translate([tx3, ty3, 0])
      if (v3r) circle(r=v3);
      else square($E, center=true);    
  }
}


module mixedRoundNGon(d, verts=[], center=false, centerX=false, centerY=false) {

  // NOT IMPLEMENTED!!!
  
  // function should be: length of verts array describes number of edge, items describe their radius
  
  // not shure if useful…

}


/**
  *
  * TODO: Moar documentation!
  *
  */
module roundCube(sx, sy, sz, r, center=false, centerX=false, centerY=false, centerZ=false) {
  tx = center || centerX ? -sx/2 : 0;
  ty = center || centerY ? -sy/2 : 0;
  tz = center || centerZ ? -sz/2 : 0;

  translate([tx, ty, tz])
    if (r > 0)
      hull() for (_tx=[r, sx-r], _ty=[r, sy-r], _tz=[r, sz-r]) translate([_tx, _ty, _tz]) sphere(r=r);
    else
      cube([sx, sy, sz]);
}


/**
  *
  * TODO: Proper documentation!
  *
  * `d` the diameter; measured corner-to-corner for even corner numbers
  * `n` number of corners
  * `r` corner radius
  */
module roundNGon(d, n, r) {
  
  assert(d >= 0, assertDiameterMsg);
  assert(r > 0 && r < d/2, "radius out of range[ 0 < r < d/2 ]");
  assert(n > 2, "corner count out of range[ > 2 ]");
  
  minkowski() {
    circle(d=d - 2*r, $fn=n);
    if (r > 0) circle(r=r);
  }
}
