#!/usr/bin/env nix-shell
#!nix-shell -i python -p openscad "python3.withPackages(ps : with ps; [ pyyaml ])"


import yaml
import shutil
import subprocess


class OpenSCAD:

  def __init__(self):
    self.__openscad = shutil.which('openscad')

  def __call__(self, *args, outfile=None, csglimit=None, **kwargs):
    '''args are added at the end, kwargs are used to add variable overrides'''

    cmd = [self.__openscad]


    # Can we do better instead of replicating this all over the place???

    if csglimit is not None:
      cmd += [f'--csglimit={csglimit}']


    for k, v in kwargs.items():
      cmd +=  ['-D', f'{k}="{v}"']


    if outfile is not None:
      cmd += ['-o', outfile]

    cmd += args

    print(f'Running OpenSCAD: {cmd}')  # TODO: remove after debugging?!
    return subprocess.Popen(cmd).communicate()


def main():
  # TODO: Error check for missing files!
  with open('parts.yml', 'r') as f:
    parts = yaml.safe_load(f)

  # print(parts)

  scad = OpenSCAD()

  for part in parts:
    # TODO: error check for missing attributes!

    name = part['name']
    file = part['file']
    outfile = part['outfile']

    del part['name']
    del part['file']
    del part['outfile']

    vars = part
    vars['$part'] = name

    scad(file, outfile=outfile, **vars)















if __name__ == '__main__':
  main()
